package com.tidecloud.dataacceptance.service;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.support.SendResult;
import org.springframework.util.concurrent.FailureCallback;
import org.springframework.util.concurrent.ListenableFuture;
import org.springframework.util.concurrent.SuccessCallback;

import com.smartsanitation.common.util.StringUtil;
import com.tidecloud.dataacceptance.entity.ConnectMsg;

import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;

public abstract class AcceptanceInboundHandlerAdapter extends ChannelInboundHandlerAdapter {
	
	protected String ip;
	
	protected Integer port;

	protected String dataPath;

	protected String prefixName ;
	
	private static final Logger logger = LoggerFactory.getLogger(AcceptanceInboundHandlerAdapter.class);
	private static final Long TEN_M = 10485760l;
	
	private static String PREFIX_LINK = "s.";
	private static String PREFIX_DEVICE = "d.";

	public static Map<String, String> channelMap = new HashMap<String, String>();
	public static Map<String, Channel> manageChannelMap = new HashMap<>();
	public static Map<String, Channel> channelMapOfChannelKey = new HashMap<>();
	public static Map<String, String> commandCopy = new HashMap<>();
	private static ExecutorService singleThreadPool = Executors.newSingleThreadExecutor();

	private static ExecutorService kafkaSendThreadPool = Executors.newSingleThreadExecutor();
	@Autowired
	private JedisPool jedisPool;
	@Autowired
	private KafkaTemplate<Integer, String> kafkaTemplate;

	private String topic;


	@Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
        ByteBuf byteBuf = (ByteBuf)msg;
        String str = byteBufferToString(byteBuf.nioBuffer());
        logger.info("上传数据：{}", str);
       
        try {
            reply(ctx, str);
        } catch (Exception e) {
            logger.error(e.getMessage());
        } finally {
        	byteBuf.release();
        }
    }
	abstract public void reply(ChannelHandlerContext ctx, String msg) throws Exception;
	
	protected void sendMsg(String msg,String deviceId) {
    	kafkaSendThreadPool.execute(()->sendKakfaMsg(msg,deviceId));
        singleThreadPool.execute(()->dataStorage(msg));
    }



	public void manageLink(Channel channel, String deviceId) {
		String channelId = UUID.randomUUID().toString();
		manageChannelMap.put(channelId, channel);
		channelMap.put(channelId, deviceId);
		channelMapOfChannelKey.put(deviceId, channel);
		logger.info("链接管理,链接id [{}]", channelId);

		ConnectMsg cMsg = new ConnectMsg(this.ip, channelId);
		try (Jedis jedis = jedisPool.getResource()) {
			jedis.select(15);
			String insertKey = PREFIX_LINK + ip;
			String selectKey = PREFIX_DEVICE + deviceId;

			jedis.set(insertKey, deviceId);
			jedis.set(selectKey, StringUtil.convert2String(cMsg));
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage());
		}
	}




	protected void sendKakfaMsg(String msg, String deviceId) {
		Integer key = new Integer(deviceId.hashCode());
		ListenableFuture<SendResult<Integer, String>> listenableFuture = kafkaTemplate.send(topic, key, msg);
		
		// 发送成功回调
		SuccessCallback<SendResult<Integer, String>> successCallback = new SuccessCallback<SendResult<Integer, String>>() {
			@Override
			public void onSuccess(SendResult<Integer, String> result) {
				// 成功业务逻辑
				logger.info("发送kafka成功.deviceId:{}, msg:{}", key, msg);
			}
		};
		// 发送失败回调
		FailureCallback failureCallback = new FailureCallback() {
			@Override
			public void onFailure(Throwable ex) {
				// 失败业务逻辑
				logger.error("发送kafka失败.deviceId:{}, msg:{}", key, msg);
			}
		};
		listenableFuture.addCallback(successCallback, failureCallback);

	}

	public void dataStorage(String deviceStr) {
		File path = new File(dataPath);
		File[] listFiles = path.listFiles();
		boolean isTouch = true;
		if (listFiles != null) {
			for (File sonFile : listFiles) {
				long len = sonFile.length();
				if (len < TEN_M) {
					// String fileName = dataPath + prefixName + new
					// SimpleDateFormat("yyMMddHHmmss").format(new Date());
					// File file = new File(fileName);
					writeDevice2File(sonFile, deviceStr);
					logger.info("正在写入数据: " + deviceStr);
					isTouch = false;
				}
			}
		}
		if (isTouch) {
			String fileName = dataPath + prefixName + new SimpleDateFormat("yyMMddHHmmss").format(new Date());
			File file = new File(fileName);
			writeDevice2File(file, deviceStr);
			logger.info("满10M，创建新的文件 正在写入数据：" + deviceStr + "timestamp："
					+ new SimpleDateFormat("yyMMddHHmmss").format(new Date()));
		}
	}

	public static void writeDevice2File(File file, String deviceStr) {
		BufferedWriter bufferedWriter = null;
		try {
			bufferedWriter = new BufferedWriter(new FileWriter(file, true));
			bufferedWriter.write(deviceStr);
			bufferedWriter.newLine();
			bufferedWriter.flush();
		} catch (IOException e) {
			logger.error(e.getMessage());
		} finally {
			if (bufferedWriter != null) {
				try {
					bufferedWriter.close();
				} catch (IOException e) {
					logger.error("close file[{}] failed: {}", file.getName(), e.getMessage());
				}
			}

		}
	}

	@Override
	public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
		cause.printStackTrace();
		ctx.close();
	}

	@Override
	public void channelActive(final ChannelHandlerContext ctx) throws Exception {
		saveChannel(ctx);
	}

	private void saveChannel(ChannelHandlerContext ctx) {
	}

	public static String byteBufferToString(ByteBuffer buffer) {
		CharBuffer charBuffer = null;
		try {
			Charset charset = Charset.forName("UTF-8");
			CharsetDecoder decoder = charset.newDecoder();
			charBuffer = decoder.decode(buffer);
			buffer.flip();
			return charBuffer.toString();
		} catch (Exception ex) {
			ex.printStackTrace();
			return null;
		}
	}

	protected Integer getInteger(String str) {
		if (str == null) {
			return null;
		}
		return Integer.valueOf(str);
	}

	protected Double getDouble(String str) {
		if (str == null) {
			return null;
		}
		return Double.valueOf(str);
	}

	static byte[] int2bytes(int num) {
		byte[] b = new byte[4];
		// int mask=0xff;
		for (int i = 0; i < 4; i++) {
			b[3 - i] = (byte) (num >>> (24 - i * 8));
		}
		return b;
	}

	public String getDataPath() {
		return dataPath;
	}

	public void setDataPath(String dataPath) {
		this.dataPath = dataPath;
	}

	public String getTopic() {
		return topic;
	}

	public void setTopic(String topic) {
		this.topic = topic;
	}
	public String getIp() {
		return ip;
	}
	public void setIp(String ip) {
		this.ip = ip;
	}
	public Integer getPort() {
		return port;
	}
	public void setPort(Integer port) {
		this.port = port;
	}
	public String getPrefixName() {
		return prefixName;
	}
	public void setPrefixName(String prefixName) {
		this.prefixName = prefixName;
	}

}
